#pragma once

namespace mimosa
{
  namespace rpc
  {
    class MissingRequiredField
    {
    };

    class InvalidFormat
    {
    };

    class FieldAlreadySet
    {
    };

    class InvalidEnumValue
    {
    };
  }
}

