#pragma once

namespace mimosa {
   namespace bencode {
   }
   namespace format {
   }
   namespace http {
   }
   namespace log {
   }
   namespace net {
   }
   namespace options {
   }
   namespace rpc {
   }
   namespace stream {
   }
   namespace tpl {
   }
} // namespace mimosa

namespace hefur {
   namespace m = mimosa;
   namespace mb = mimosa::bencode;
   namespace mf = mimosa::format;
   namespace mh = mimosa::http;
   namespace ml = mimosa::log;
   namespace mn = mimosa::net;
   namespace mo = mimosa::options;
   namespace mr = mimosa::rpc;
   namespace ms = mimosa::stream;
   namespace mt = mimosa::tpl;
} // namespace hefur
